@@ -1,7 +1,7 @@
 # Author: David Goodger
 # Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 1.42 $
-# Date: $Date: 2003/06/03 02:17:27 $
+# Revision: $Revision: 1.56 $
+# Date: $Date: 2004/06/23 02:25:34 $
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -18,7 +18,7 @@ is represented by abstract base classes (`Root`, `Structural`, `Body`,
 ``isinstance(node, base_class)`` to determine the position of the node in the
 hierarchy.
 
-.. _DTD: http://docutils.sourceforge.net/spec/docutils.dtd
+.. _DTD: http://docutils.sourceforge.net/docs/ref/docutils.dtd
 """
 
 __docformat__ = 'reStructuredText'
@@ -299,7 +299,7 @@ class Element(Node):
             return '<%s...>' % self.tagname
 
     def __str__(self):
-        return unicode(self).encode('raw_unicode_escape')
+        return self.__unicode__().encode('raw_unicode_escape')
 
     def __unicode__(self):
         if self.children:
@@ -512,7 +512,14 @@ class TextElement(Element):
     """
     An element which directly contains text.
 
-    Its children are all Text or TextElement nodes.
+    Its children are all `Text` or `TextElement` subclass nodes.  You can
+    check whether an element's context is inline simply by checking whether
+    its immediate parent is a `TextElement` instance (including subclasses).
+    This is handy for nodes like `image` that can appear both inline and as
+    standalone body elements.
+
+    If passing children to `__init__()`, make sure to set `text` to
+    ``''`` or some other suitable value.
     """
 
     child_text_separator = ''
@@ -595,6 +602,9 @@ class Targetable(Resolvable):
 
     referenced = 0
 
+    indirect_reference_name = None
+    """Holds the whitespace_normalized_name (contains mixed case) of a target"""
+
 class Labeled:
     """Contains a `label` as its first element."""
 
@@ -816,6 +826,7 @@ class document(Root, Structural, Element):
     def has_name(self, name):
         return self.nameids.has_key(name)
 
+    # "note" here is an imperative verb: "take note of".
     def note_implicit_target(self, target, msgnode=None):
         id = self.set_id(target, msgnode)
         self.set_name_id_map(target, id, msgnode, explicit=None)
@@ -935,6 +946,7 @@ class rubric(Titular, TextElement): pass
 # ========================
 
 class docinfo(Bibliographic, Element): pass
+class info(Bibliographic, Element): pass
 class author(Bibliographic, TextElement): pass
 class authors(Bibliographic, Element): pass
 class organization(Bibliographic, TextElement): pass
@@ -1111,8 +1123,8 @@ class pending(Special, Invisible, PreBibliographic, Element):
 
     But the "contents" directive can't do its work until the entire document
     has been parsed and possibly transformed to some extent.  So the directive
-    code leaves a placeholder behind that will trigger the second phase of the
-    its processing, something like this::
+    code leaves a placeholder behind that will trigger the second phase of its
+    processing, something like this::
 
         <pending ...public attributes...> + internal attributes
 
@@ -1178,7 +1190,7 @@ class raw(Special, Inline, PreBibliographic, FixedTextElement):
 class emphasis(Inline, TextElement): pass
 class strong(Inline, TextElement): pass
 class literal(Inline, TextElement): pass
-class reference(Inline, Referential, TextElement): pass
+class reference(General, Inline, Referential, TextElement): pass
 class footnote_reference(Inline, Referential, TextElement): pass
 class citation_reference(Inline, Referential, TextElement): pass
 class substitution_reference(Inline, TextElement): pass
@@ -1218,7 +1230,7 @@ node_class_names = """
         footnote footnote_reference
     generated
     header hint
-    image important inline
+    image important info inline
     label legend line_block list_item literal literal_block
     note
     option option_argument option_group option_list option_list_item
@@ -1269,8 +1281,8 @@ class NodeVisitor:
 
         Raise an exception unless overridden.
         """
-        raise NotImplementedError('visiting unknown node type: %s'
-                                  % node.__class__.__name__)
+        raise NotImplementedError('%s visiting unknown node type: %s'
+                                  % (self.__class__, node.__class__.__name__))
 
     def unknown_departure(self, node):
         """
@@ -1278,8 +1290,8 @@ class NodeVisitor:
 
         Raise exception unless overridden.
         """
-        raise NotImplementedError('departing unknown node type: %s'
-                                  % node.__class__.__name__)
+        raise NotImplementedError('%s departing unknown node type: %s'
+                                  % (self.__class__, node.__class__.__name__))
 
 
 class SparseNodeVisitor(NodeVisitor):
@@ -1291,13 +1303,6 @@ class SparseNodeVisitor(NodeVisitor):
     subclasses), subclass `NodeVisitor` instead.
     """
 
-    # Save typing with dynamic definitions.
-    for name in node_class_names:
-        exec """def visit_%s(self, node): pass\n""" % name
-        exec """def depart_%s(self, node): pass\n""" % name
-    del name
-
-
 class GenericNodeVisitor(NodeVisitor):
 
     """
@@ -1324,14 +1329,24 @@ class GenericNodeVisitor(NodeVisitor):
         """Override for generic, uniform traversals."""
         raise NotImplementedError
 
-    # Save typing with dynamic definitions.
-    for name in node_class_names:
-        exec """def visit_%s(self, node):
-                    self.default_visit(node)\n""" % name
-        exec """def depart_%s(self, node):
-                    self.default_departure(node)\n""" % name
-    del name
+def _call_default_visit(self, node):
+    self.default_visit(node)
 
+def _call_default_departure(self, node):
+    self.default_departure(node)
+
+def _nop(self, node):
+    pass
+
+def _add_node_class_names(names):
+    """Save typing with dynamic assignments:"""
+    for _name in names:
+        setattr(GenericNodeVisitor, "visit_" + _name, _call_default_visit)
+        setattr(GenericNodeVisitor, "depart_" + _name, _call_default_departure)
+        setattr(SparseNodeVisitor, 'visit_' + _name, _nop)
+        setattr(SparseNodeVisitor, 'depart' + _name, _nop)
+
+_add_node_class_names(node_class_names)
 
 class TreeCopyVisitor(GenericNodeVisitor):
 
