@@ -1,7 +1,7 @@
 # Author: David Goodger
 # Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 1.28 $
-# Date: $Date: 2003/06/16 21:29:34 $
+# Revision: $Revision: 1.35 $
+# Date: $Date: 2004/06/20 16:03:17 $
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -20,8 +20,12 @@ from docutils import frontend, nodes
 
 class SystemMessage(ApplicationError):
 
-    def __init__(self, system_message):
+    def __init__(self, system_message, level):
         Exception.__init__(self, system_message.astext())
+        self.level = level
+
+
+class SystemMessagePropagation(ApplicationError): pass
 
 
 class Reporter:
@@ -51,7 +55,7 @@ class Reporter:
     is compared to the thresholds stored in the category, and a warning or
     error is generated as appropriate.  Debug messages are produced iff the
     stored debug switch is on.  Message output is sent to the stored warning
-    stream.
+    stream if not set to ''.
 
     The default category is '' (empty string).  By convention, Writers should
     retrieve reporting conditions from the 'writer' category (which, unless
@@ -88,7 +92,8 @@ class Reporter:
               exceptions will be raised, halting execution.
             - `debug`: Show debug (level=0) system messages?
             - `stream`: Where warning output is sent.  Can be file-like (has a
-              ``.write`` method), a string (file name, opened for writing), or
+              ``.write`` method), a string (file name, opened for writing), 
+              '' (empty string, for discarding all stream messages) or
               `None` (implies `sys.stderr`; default).
             - `encoding`: The encoding for stderr output.
             - `error_handler`: The error handler for stderr output encoding.
@@ -99,7 +104,12 @@ class Reporter:
         if stream is None:
             stream = sys.stderr
         elif type(stream) in (StringType, UnicodeType):
-            raise NotImplementedError('This should open a file for writing.')
+            # Leave stream untouched if it's ''.
+            if stream != '':
+                if type(stream) == StringType:
+                    stream = open(stream, 'w')
+                elif type(stream) == UnicodeType:
+                    stream = open(stream.encode(), 'w')
 
         self.encoding = encoding
         """The character encoding for the stderr output."""
@@ -174,14 +184,14 @@ class Reporter:
                                    type=self.levels[level],
                                    *children, **attributes)
         debug, report_level, halt_level, stream = self[category].astuple()
-        if level >= report_level or debug and level == 0:
+        if (level >= report_level or debug and level == 0) and stream:
             msgtext = msg.astext().encode(self.encoding, self.error_handler)
             if category:
                 print >>stream, msgtext, '[%s]' % category
             else:
                 print >>stream, msgtext
         if level >= halt_level:
-            raise SystemMessage(msg)
+            raise SystemMessage(msg, level)
         if level > 0 or debug:
             self.notify_observers(msg)
         self.max_level = max(level, self.max_level)
@@ -264,6 +274,8 @@ def extract_extension_options(field_list, options_spec):
         - `KeyError` for unknown option names.
         - `ValueError` for invalid option values (raised by the conversion
            function).
+        - `TypeError` for invalid option value types (raised by conversion
+           function).
         - `DuplicateOptionError` for duplicate options.
         - `BadOptionError` for invalid fields.
         - `BadOptionDataError` for invalid option data (missing name,
@@ -320,10 +332,14 @@ def assemble_option_dict(option_list, options_spec):
         - `DuplicateOptionError` for duplicate options.
         - `ValueError` for invalid option values (raised by conversion
            function).
+        - `TypeError` for invalid option value types (raised by conversion
+           function).
     """
     options = {}
     for name, value in option_list:
-        convertor = options_spec[name]       # raises KeyError if unknown
+        convertor = options_spec[name]  # raises KeyError if unknown
+        if convertor is None:
+            raise KeyError(name)        # or if explicitly disabled
         if options.has_key(name):
             raise DuplicateOptionError('duplicate option "%s"' % name)
         try:
@@ -405,7 +421,7 @@ def clean_rcs_keywords(paragraph, keyword_substitutions):
     if len(paragraph) == 1 and isinstance(paragraph[0], nodes.Text):
         textnode = paragraph[0]
         for pattern, substitution in keyword_substitutions:
-            match = pattern.match(textnode.data)
+            match = pattern.search(textnode.data)
             if match:
                 textnode.data = pattern.sub(substitution, textnode.data)
                 return
