@@ -1,7 +1,7 @@
 # Authors:  David Goodger; Garth Kidd
 # Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 1.22 $
-# Date: $Date: 2003/06/03 02:14:28 $
+# Revision: $Revision: 1.42 $
+# Date: $Date: 2004/07/25 02:30:05 $
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -29,8 +29,12 @@ Exports the following:
     - `GridTableParserTestCase`
     - `SimpleTableParserTestSuite`
     - `SimpleTableParserTestCase`
-    - 'LatexPublishTestSuite'
-    - 'LatexPublishTestCase'
+    - `WriterPublishTestCase`
+    - `LatexWriterPublishTestCase`
+    - `PseudoXMLWriterPublishTestCase`
+    - `HtmlWriterPublishTestCase`
+    - `PublishTestSuite`
+    - `HtmlFragmentTestSuite`
     - `DevNull` (output sink)
 """
 __docformat__ = 'reStructuredText'
@@ -38,7 +42,7 @@ __docformat__ = 'reStructuredText'
 import sys
 import os
 import unittest
-import difflib
+import docutils_difflib
 import inspect
 from pprint import pformat
 from types import UnicodeType
@@ -48,13 +52,15 @@ import docutils.core
 from docutils import frontend, nodes, statemachine, urischemes, utils
 from docutils.transforms import universal
 from docutils.parsers import rst
-from docutils.parsers.rst import states, tableparser, directives, languages
-from docutils.readers import standalone, pep, python
-from docutils.statemachine import string2lines
+from docutils.parsers.rst import states, tableparser, roles, languages
+from docutils.readers import standalone, pep
+from docutils.statemachine import StringList, string2lines
 
 try:
     from docutils.readers.python import moduleparser
-except:
+    from tokenize import generate_tokens
+    del generate_tokens
+except ImportError:      # moduleparser depends on modules added in Python 2.2
     moduleparser = None
 
 try:
@@ -63,6 +69,9 @@ except:
     import pdb
 
 
+# Hack to make repr(StringList) look like repr(list):
+StringList.__repr__ = StringList.__str__
+
 
 class DevNull:
 
@@ -147,10 +156,13 @@ class CustomTestSuite(unittest.TestSuite):
         self.addTest(tc)
         return tc
 
+    def generate_no_tests(self, *args, **kwargs):
+        pass
+
 
 class CustomTestCase(unittest.TestCase):
 
-    compare = difflib.Differ().compare
+    compare = docutils_difflib.Differ().compare
     """Comparison method shared by all subclasses."""
 
     def __init__(self, method_name, input, expected, id,
@@ -267,6 +279,7 @@ class TransformTestCase(CustomTestCase):
     settings.halt_level = 5
     settings.debug = package_unittest.debug
     settings.warning_stream = DevNull()
+    unknown_reference_resolvers = ()
 
     def __init__(self, *args, **kwargs):
         self.transforms = kwargs['transforms']
@@ -326,7 +339,7 @@ class ParserTestCase(CustomTestCase):
     parser = rst.Parser()
     """Parser shared by all ParserTestCases."""
 
-    option_parser = frontend.OptionParser(components=(parser,))
+    option_parser = frontend.OptionParser(components=(rst.Parser,))
     settings = option_parser.get_default_values()
     settings.report_level = 5
     settings.halt_level = 5
@@ -336,6 +349,8 @@ class ParserTestCase(CustomTestCase):
         if self.run_in_debugger:
             pdb.set_trace()
         document = utils.new_document('test data', self.settings)
+        # Remove any additions made by "role" directives:
+        roles._roles = {}
         self.parser.parse(self.input, document)
         output = document.pformat()
         self.compare_output(self.input, output, self.expected)
@@ -383,10 +398,10 @@ class PEPParserTestCase(ParserTestCase):
 
     """PEP-specific parser test case."""
 
-    parser = rst.Parser(rfc2822=1, inliner=pep.Inliner())
+    parser = rst.Parser(rfc2822=1, inliner=rst.states.Inliner())
     """Parser shared by all PEPParserTestCases."""
 
-    option_parser = frontend.OptionParser(components=(parser, pep.Reader))
+    option_parser = frontend.OptionParser(components=(rst.Parser, pep.Reader))
     settings = option_parser.get_default_values()
     settings.report_level = 5
     settings.halt_level = 5
@@ -405,7 +420,7 @@ class GridTableParserTestCase(CustomTestCase):
     parser = tableparser.GridTableParser()
 
     def test_parse_table(self):
-        self.parser.setup(string2lines(self.input))
+        self.parser.setup(StringList(string2lines(self.input), 'test data'))
         try:
             self.parser.find_head_body_sep()
             self.parser.parse_table()
@@ -417,7 +432,8 @@ class GridTableParserTestCase(CustomTestCase):
 
     def test_parse(self):
         try:
-            output = self.parser.parse(string2lines(self.input))
+            output = self.parser.parse(StringList(string2lines(self.input),
+                                                  'test data'))
         except Exception, details:
             output = '%s: %s' % (details.__class__.__name__, details)
         self.compare_output(self.input, pformat(output) + '\n',
@@ -510,11 +526,11 @@ class PythonModuleParserTestCase(CustomTestCase):
     def test_parser(self):
         if self.run_in_debugger:
             pdb.set_trace()
-        module = moduleparser.parse_module(self.input, 'test data')
+        module = moduleparser.parse_module(self.input, 'test data').pformat()
         output = str(module)
         self.compare_output(self.input, output, self.expected)
 
-    def test_token_parser_rhs(self): 
+    def test_token_parser_rhs(self):
         if self.run_in_debugger:
             pdb.set_trace()
         tr = moduleparser.TokenParser(self.input)
@@ -528,10 +544,16 @@ class PythonModuleParserTestSuite(CustomTestSuite):
     A collection of PythonModuleParserTestCase.
     """
 
-    if moduleparser is None:
-        PythonModuleParserTestCase.test_parser = CustomTestCase.skip_test
-        PythonModuleParserTestCase.test_token_parser_rhs = \
-            CustomTestCase.skip_test
+    notified = None
+
+    def __init__(self, *args, **kwargs):
+        if moduleparser is None:
+            if not self.notified:
+                print ('Tests of docutils.readers.python skipped; '
+                       'Python 2.2 or higher required.')
+                PythonModuleParserTestSuite.notified = 1
+            self.generateTests = self.generate_no_tests
+        CustomTestSuite.__init__(self, *args, **kwargs)
 
     def generateTests(self, dict, dictname='totest',
                       testmethod='test_parser'):
@@ -560,15 +582,14 @@ class PythonModuleParserTestSuite(CustomTestSuite):
                       run_in_debugger=run_in_debugger)
 
 
-# @@@ These should be generalized to WriterPublishTestCase/Suite or
-# just PublishTestCase/Suite, as per TransformTestCase/Suite.
-class LatexPublishTestCase(CustomTestCase, docutils.SettingsSpec):
+class WriterPublishTestCase(CustomTestCase, docutils.SettingsSpec):
 
     """
     Test case for publish.
     """
 
     settings_default_overrides = {'_disable_config': 1}
+    writer_name = '' # override in subclasses
 
     def test_publish(self):
         if self.run_in_debugger:
@@ -577,15 +598,43 @@ class LatexPublishTestCase(CustomTestCase, docutils.SettingsSpec):
               source=self.input,
               reader_name='standalone',
               parser_name='restructuredtext',
-              writer_name='latex',
+              writer_name=self.writer_name,
               settings_spec=self)
         self.compare_output(self.input, output, self.expected)
 
 
-class LatexPublishTestSuite(CustomTestSuite):
+class LatexWriterPublishTestCase(WriterPublishTestCase):
 
-    def __init__(self):
+    """
+    Test case for Latex writer.
+    """
+
+    writer_name = 'latex'
+
+
+class PseudoXMLWriterPublishTestCase(WriterPublishTestCase):
+
+    """
+    Test case for pseudo-XML writer.
+    """
+
+    writer_name = 'pseudoxml'
+
+
+class PublishTestSuite(CustomTestSuite):
+
+    TEST_CLASSES = {
+        'latex': LatexWriterPublishTestCase,
+        'pseudoxml': PseudoXMLWriterPublishTestCase,
+    }
+
+    def __init__(self, writer_name):
+        """
+        `writer_name` is the name of the writer
+        to use.  It must be a key in `TEST_CLASSES`.
+        """
         CustomTestSuite.__init__(self)
+        self.test_class = self.TEST_CLASSES[writer_name]
 
     def generateTests(self, dict, dictname='totest'):
         for name, cases in dict.items():
@@ -598,14 +647,101 @@ class LatexPublishTestSuite(CustomTestSuite):
                     else:
                         continue
                 self.addTestCase(
-                      LatexPublishTestCase, 'test_publish',
+                      self.test_class, 'test_publish',
                       input=case[0], expected=case[1],
                       id='%s[%r][%s]' % (dictname, name, casenum),
                       run_in_debugger=run_in_debugger)
 
 
-def exception_args(code):
+class HtmlPublishPartsTestSuite(CustomTestSuite):
+
+    def generateTests(self, dict, dictname='totest'):
+        for name, (settings_overrides, cases) in dict.items():
+            for casenum in range(len(cases)):
+                case = cases[casenum]
+                run_in_debugger = 0
+                if len(case)==3:
+                    if case[2]:
+                        run_in_debugger = 1
+                    else:
+                        continue
+                self.addTestCase(
+                      HtmlWriterPublishPartsTestCase, 'test_publish',
+                      settings_overrides=settings_overrides,
+                      input=case[0], expected=case[1],
+                      id='%s[%r][%s]' % (dictname, name, casenum),
+                      run_in_debugger=run_in_debugger)
+
+
+class HtmlWriterPublishPartsTestCase(WriterPublishTestCase):
+
+    """
+    Test case for HTML writer via the publish_parts interface.
+    """
+
+    writer_name = 'html'
+
+    def __init__(self, *args, **kwargs):
+        self.settings_overrides = kwargs['settings_overrides']
+        """Settings overrides to use for this test case."""
+
+        del kwargs['settings_overrides'] # only wanted here
+        CustomTestCase.__init__(self, *args, **kwargs)
+
+    def test_publish(self):
+        if self.run_in_debugger:
+            pdb.set_trace()
+        parts = docutils.core.publish_parts(
+            source=self.input,
+            reader_name='standalone',
+            parser_name='restructuredtext',
+            writer_name=self.writer_name,
+            settings_spec=self,
+            settings_overrides=self.settings_overrides)
+        output = self.format_output(parts)
+        # interpolate standard variables:
+        expected = self.expected % {'version': docutils.__version__}
+        self.compare_output(self.input, output, expected)
+
+    standard_meta_value = """\
+<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
+<meta name="generator" content="Docutils %s: http://docutils.sourceforge.net/" />
+""" % docutils.__version__
+    standard_stylesheet_value = ('<link rel="stylesheet" href="default.css" '
+                                 'type="text/css" />\n')
+
+    def format_output(self, parts):
+        """Minimize & standardize the output."""
+        # remove redundant bits:
+        del parts['whole']
+        del parts['body']
+        # remove standard bits:
+        parts['meta'] = parts['meta'].replace(self.standard_meta_value, '')
+        if parts['stylesheet'] == self.standard_stylesheet_value:
+            del parts['stylesheet']
+        # remove empty values:
+        for key in parts.keys():
+            if not parts[key]:
+                del parts[key]
+        # standard output format:
+        keys = parts.keys()
+        keys.sort()
+        output = []
+        for key in keys:
+            output.append("%r: '''%s'''"
+                          % (key, parts[key].encode('raw_unicode_escape')))
+            if output[-1].endswith("\n'''"):
+                output[-1] = output[-1][:-4] + "\\n'''"
+        return '{' + ',\n '.join(output) + '}\n'
+
+
+def exception_data(code):
+    """
+    Execute `code` and return the resulting exception, the exception arguments,
+    and the formatted exception string.
+    """
     try:
         exec(code)
     except Exception, detail:
-        return detail.args
+        return (detail, detail.args,
+                '%s: %s' % (detail.__class__.__name__, detail))
