@@ -2,8 +2,8 @@
 
 # Author: David Goodger
 # Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 1.15 $
-# Date: $Date: 2003/03/22 06:02:17 $
+# Revision: $Revision: 1.21 $
+# Date: $Date: 2004/06/28 19:55:26 $
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -280,7 +280,7 @@ ref_
 """\
 <document source="test data">
     <paragraph>
-        <reference refname="ref">
+        <reference name="ref" refname="ref">
             ref
 """],
 ["""\
@@ -289,7 +289,7 @@ ref__
 """\
 <document source="test data">
     <paragraph>
-        <reference anonymous="1">
+        <reference anonymous="1" name="ref">
             ref
 """],
 ["""\
@@ -299,19 +299,19 @@ but not _ref_ or __attr__ or object.__attr__
 """\
 <document source="test data">
     <paragraph>
-        <reference refname="ref">
+        <reference name="ref" refname="ref">
             ref
         , \n\
-        <reference refname="r">
+        <reference name="r" refname="r">
             r
         , \n\
-        <reference refname="r_e-f">
+        <reference name="r_e-f" refname="r_e-f">
             r_e-f
         , -
-        <reference refname="ref">
+        <reference name="ref" refname="ref">
             ref
         , and \n\
-        <reference anonymous="1">
+        <reference anonymous="1" name="anonymousref">
             anonymousref
         ,
         but not _ref_ or __attr__ or object.__attr__
@@ -325,7 +325,7 @@ totest['phrase_references'] = [
 """\
 <document source="test data">
     <paragraph>
-        <reference refname="phrase reference">
+        <reference name="phrase reference" refname="phrase reference">
             phrase reference
 """],
 ["""\
@@ -334,7 +334,7 @@ totest['phrase_references'] = [
 """\
 <document source="test data">
     <paragraph>
-        <reference anonymous="1">
+        <reference anonymous="1" name="anonymous reference">
             anonymous reference
 """],
 ["""\
@@ -344,7 +344,7 @@ across lines`_
 """\
 <document source="test data">
     <paragraph>
-        <reference refname="phrase reference across lines">
+        <reference name="phrase reference across lines" refname="phrase reference across lines">
             phrase reference
             across lines
 """],
@@ -354,7 +354,7 @@ across lines`_
 """\
 <document source="test data">
     <paragraph>
-        <reference refname="phrase`_ reference">
+        <reference name="phrase`_ reference" refname="phrase`_ reference">
             phrase`_ reference
 """],
 ["""\
@@ -398,7 +398,7 @@ Invalid phrase reference:
         <problematic id="id2" refid="id1">
             `
         phrase \n\
-        <reference refname="reference">
+        <reference name="reference" refname="reference">
             reference
          without closing backquote
     <system_message backrefs="id2" id="id1" level="2" line="1" source="test data" type="WARNING">
@@ -414,7 +414,7 @@ Invalid phrase reference:
         <problematic id="id2" refid="id1">
             `
         anonymous phrase \n\
-        <reference anonymous="1">
+        <reference anonymous="1" name="reference">
             reference
          without closing backquote
     <system_message backrefs="id2" id="id1" level="2" line="1" source="test data" type="WARNING">
@@ -430,7 +430,7 @@ totest['embedded_URIs'] = [
 """\
 <document source="test data">
     <paragraph>
-        <reference refuri="http://example.com">
+        <reference name="phrase reference" refuri="http://example.com">
             phrase reference
         <target id="phrase-reference" name="phrase reference" refuri="http://example.com">
 """],
@@ -440,7 +440,7 @@ totest['embedded_URIs'] = [
 """\
 <document source="test data">
     <paragraph>
-        <reference refuri="http://example.com">
+        <reference name="anonymous reference" refuri="http://example.com">
             anonymous reference
 """],
 ["""\
@@ -450,7 +450,7 @@ totest['embedded_URIs'] = [
 """\
 <document source="test data">
     <paragraph>
-        <reference refuri="http://example.com">
+        <reference name="embedded URI on next line" refuri="http://example.com">
             embedded URI on next line
 """],
 ["""\
@@ -460,7 +460,7 @@ long/path>`__
 """\
 <document source="test data">
     <paragraph>
-        <reference refuri="http://example.com/long/path">
+        <reference name="embedded URI across lines" refuri="http://example.com/long/path">
             embedded URI across lines
 """],
 ["""\
@@ -470,7 +470,7 @@ long/path /and  /whitespace>`__
 """\
 <document source="test data">
     <paragraph>
-        <reference refuri="http://example.com/long/path/and/whitespace">
+        <reference name="embedded URI with whitespace" refuri="http://example.com/long/path/and/whitespace">
             embedded URI with whitespace
 """],
 ["""\
@@ -482,10 +482,10 @@ long/path /and  /whitespace>`__
 """\
 <document source="test data">
     <paragraph>
-        <reference refuri="mailto:jdoe@example.com">
+        <reference name="embedded email address" refuri="mailto:jdoe@example.com">
             embedded email address
     <paragraph>
-        <reference refuri="mailto:jdoe@example.com">
+        <reference name="embedded email address broken across lines" refuri="mailto:jdoe@example.com">
             embedded email address broken across lines
 """],
 ["""\
@@ -504,25 +504,44 @@ See `HTML Anchors: \\<a>`_.
 """\
 <document source="test data">
     <paragraph>
-        <reference anonymous="1">
+        <reference anonymous="1" name="embedded URI with too much whitespace < http://example.com/ long/path /and /whitespace >">
             embedded URI with too much whitespace < http://example.com/
             long/path /and  /whitespace >
     <paragraph>
-        <reference anonymous="1">
+        <reference anonymous="1" name="embedded URI with too much whitespace at end <http://example.com/ long/path /and /whitespace >">
             embedded URI with too much whitespace at end <http://example.com/
             long/path /and  /whitespace >
     <paragraph>
-        <reference anonymous="1">
+        <reference anonymous="1" name="embedded URI with no preceding whitespace<http://example.com>">
             embedded URI with no preceding whitespace<http://example.com>
     <paragraph>
-        <reference anonymous="1">
+        <reference anonymous="1" name="escaped URI <http://example.com>">
             escaped URI <http://example.com>
     <paragraph>
         See \n\
-        <reference refname="html anchors: <a>">
+        <reference name="HTML Anchors: <a>" refname="html anchors: <a>">
             HTML Anchors: <a>
         .
 """],
+["""\
+Relative URIs' reference text can be omitted:
+
+`<reference>`_
+
+`<anonymous>`__
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        Relative URIs' reference text can be omitted:
+    <paragraph>
+        <reference name="reference" refuri="reference">
+            reference
+        <target id="reference" name="reference" refuri="reference">
+    <paragraph>
+        <reference name="anonymous" refuri="anonymous">
+            anonymous
+"""],
 ]
 
 totest['inline_targets'] = [
@@ -803,6 +822,52 @@ ftp://ends.with.a.period.
         ?)
 """],
 ["""\
+Valid URLs with escaped markup characters:
+
+http://example.com/\\*content\\*/whatever
+
+http://example.com/\\*content*/whatever
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        Valid URLs with escaped markup characters:
+    <paragraph>
+        <reference refuri="http://example.com/*content*/whatever">
+            http://example.com/*content*/whatever
+    <paragraph>
+        <reference refuri="http://example.com/*content*/whatever">
+            http://example.com/*content*/whatever
+"""],
+["""\
+Valid URLs may end with punctuation inside "<>":
+
+<http://example.org/ends-with-dot.>
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        Valid URLs may end with punctuation inside "<>":
+    <paragraph>
+        <
+        <reference refuri="http://example.org/ends-with-dot.">
+            http://example.org/ends-with-dot.
+        >
+"""],
+["""\
+Valid URLs with interesting endings:
+
+http://example.org/ends-with-pluses++
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        Valid URLs with interesting endings:
+    <paragraph>
+        <reference refuri="http://example.org/ends-with-pluses++">
+            http://example.org/ends-with-pluses++
+"""],
+["""\
 None of these are standalone hyperlinks (their "schemes"
 are not recognized): signal:noise, a:b.
 """,
